/* eslint-disable no-template-curly-in-string */
const Promise = require('bluebird');
const CFError = require('cf-errors');
const secretStoreResolver = require('../resolvers/secret-store');
const resolver = require('../');
const { split } = require('../utils');
const errors = require('../errors');
const utils = require('../utils');

const getContextFn = mock => jest.fn(mock);

const getLogger = () => ({
    info: jest.fn(() => {}),
    warn: jest.fn(() => {}),
});

describe('Variable resolver', () => {

    it('Should return the variable as-is in case it is not in Vars format ${{.*}}', async () => {
        const vr = resolver.build();
        const expected = [
            {
                index: 0,
                key: 'test',
                value: 'test',
                resolved: false,
            }
        ];
        await expect(vr.resolve('test')).resolves.toEqual(expected);
    });

    it('Should return the variable as-is when resolver not found', async () => {
        const vr = resolver.build();
        const expected = [
            {
                index: 0,
                key: '${{test}}',
                value: '${{test}}',
                resolved: false,
            }
        ];
        await expect(vr.resolve('${{test}}')).resolves.toEqual(expected);
    });

    it('Should resolve varialbe with proper resolver', async () => {
        const vr = resolver.build({
            'test': { resolve: () => ('resolved') }
        }, { getContext: getContextFn(), log: getLogger() });
        const expected = [
            {
                index: 0,
                key: '${{test.test}}',
                value: 'resolved',
                resolved: true,
            }
        ];
        await expect(vr.resolve('${{test.test}}')).resolves.toEqual(expected);
    });

    it('Should use the provided getContext function', async () => {
        const vr = resolver.build({
            'test': {
                resolve: async (variable, getContext) => {
                    expect(variable).toEqual('${{test.test}}');
                    await expect(getContext()).resolves.toEqual('some-context');
                    return 'resolved';
                },
            }
        }, { getContext: getContextFn(() => Promise.resolve('some-context')), log: getLogger() });
        const expected = [
            {
                index: 0,
                key: '${{test.test}}',
                value: 'resolved',
                resolved: true,
            }
        ];
        await expect(vr.resolve('${{test.test}}')).resolves.toEqual(expected);
    });

    it('Should use the provided getContext function using setGetContext', async () => {
        const vr = resolver.build({
            'test': {
                resolve: async (variable, getContext) => {
                    expect(variable).toEqual('${{test.test}}');
                    await expect(getContext()).resolves.toEqual('some-context');
                    return 'resolved';
                },
            }
        }, { log: getLogger() });
        vr.setGetContext(getContextFn(() => Promise.resolve('some-context')));
        const expected = [
            {
                index: 0,
                key: '${{test.test}}',
                value: 'resolved',
                resolved: true,
            }
        ];
        await expect(vr.resolve('${{test.test}}')).resolves.toEqual(expected);
    });

    it('Should use the provided getContext function using the resolve options', async () => {
        const vr = resolver.build({
            'test': {
                resolve: async (variable, getContext) => {
                    expect(variable).toEqual('${{test.test}}');
                    await expect(getContext()).resolves.toEqual('some-context');
                    return 'resolved';
                },
            }
        }, { log: getLogger() });
        const expected = [
            {
                index: 0,
                key: '${{test.test}}',
                value: 'resolved',
                resolved: true,
            }
        ];
        await expect(vr.resolve('${{test.test}}', {
            getContext: getContextFn(() => Promise.resolve('some-context')),
        })).resolves.toEqual(expected);
    });

    it('Should resolve empty string in case the variable was empty string', async () => {
        const vr = resolver.build();
        const expected = [
            {
                index: 0,
                key: '',
                value: '',
                resolved: false,
            }
        ];
        await expect(vr.resolve('')).resolves.toEqual(expected);
    });

    it('Should resolve empty string in case the variable was not defined', async () => {
        const vr = resolver.build();
        const expected = [
            {
                index: 0,
                key: undefined,
                value: '',
                resolved: false,
            }
        ];
        await expect(vr.resolve(undefined)).resolves.toEqual(expected);
    });

    it('Should resolve variable with multiple variables', async () => {
        const vr = resolver.build({
            'test': { resolve: () => ('resolved') }
        }, { getContext: getContextFn() });
        const expected = [
            {
                index: 0,
                key: '${{test.test}}',
                value: 'resolved',
                resolved: true,
            },
            {
                index: 1,
                key: '${{test.test}}',
                value: 'resolved',
                resolved: true,
            }
        ];
        const logger = getLogger();
        await expect(vr.resolve('${{test.test}}/${{test.test}}', { log: logger })).resolves.toEqual(expected);
        expect(logger.info).toHaveBeenCalledTimes(2);
    });

    it('Should resolve only using supported resolvers when multiple variables found', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'test_1': { resolve: () => ('resolved') }
        }, { log });
        const expected = [
            {
                index: 0,
                key: '${{test_1.test}}',
                value: 'resolved',
                resolved: true,
            },
            {
                index: 1,
                key: '${{test_2.test}}',
                value: '${{test_2.test}}',
                resolved: false,
            }
        ];
        await expect(vr.resolve('${{test_1.test}}/${{test_2.test}}')).resolves.toEqual(expected);
        expect(log.info).toHaveBeenCalledTimes(1);
    });

    it('Should use the logger provided in the resolve options', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'test_1': { resolve: () => ('resolved') }
        });
        const expected = [
            {
                index: 0,
                key: '${{test_1.test}}',
                value: 'resolved',
                resolved: true,
            },
            {
                index: 1,
                key: '${{test_2.test}}',
                value: '${{test_2.test}}',
                resolved: false,
            }
        ];
        await expect(vr.resolve('${{test_1.test}}/${{test_2.test}}', { log })).resolves.toEqual(expected);
        expect(log.info).toHaveBeenCalledTimes(1);
    });

    it('Should create logs mask for the resolved variable is shouldMask for the resolver', async () => {
        const log = getLogger();
        const addNewMask = jest.fn(() => Promise.resolve());
        const vr = resolver.build({
            'test_resolver': { resolve: () => ('resolved'), secret: true },
            'test_resolver2': { resolve: () => ('resolved') }, // should not create mask
        }, { log, addNewMask });

        await vr.resolve('${{test_resolver.test}}');
        expect(addNewMask).toHaveBeenCalledTimes(1); // should create mask only for test_resolver
        expect(addNewMask.mock.calls[0][0].key).toEqual('${{test_resolver.test}}');
        expect(addNewMask.mock.calls[0][0].value).toEqual('resolved');
        expect(log.info).toHaveBeenCalledTimes(2);
    });

    it('Should use the provided addNewMask function using setAddNewMast', async () => {
        const log = getLogger();
        const addNewMask = jest.fn(() => Promise.resolve());
        const vr = resolver.build({
            'test_resolver': { resolve: () => ('resolved'), secret: true },
            'test_resolver2': { resolve: () => ('resolved') }, // should not create mask
        }, { log });
        vr.setAddNewMask(addNewMask);
        await vr.resolve('${{test_resolver.test}}');
        expect(addNewMask).toHaveBeenCalledTimes(1); // should create mask only for test_resolver
        expect(addNewMask.mock.calls[0][0].key).toEqual('${{test_resolver.test}}');
        expect(addNewMask.mock.calls[0][0].value).toEqual('resolved');
        expect(log.info).toHaveBeenCalledTimes(2);
    });

    it('Should use the provided addNewMask function using resolve options', async () => {
        const log = getLogger();
        const addNewMask = jest.fn(() => Promise.resolve());
        const vr = resolver.build({
            'test_resolver': { resolve: () => ('resolved'), secret: true },
            'test_resolver2': { resolve: () => ('resolved') }, // should not create mask
        }, { log });
        await vr.resolve('${{test_resolver.test}}', { addNewMask });
        expect(addNewMask).toHaveBeenCalledTimes(1); // should create mask only for test_resolver
        expect(addNewMask.mock.calls[0][0].key).toEqual('${{test_resolver.test}}');
        expect(addNewMask.mock.calls[0][0].value).toEqual('resolved');
        expect(log.info).toHaveBeenCalledTimes(2);
    });

    it('Should throw an error in case the resoution flow throws critical error', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'test_1': { resolve: () => {
                throw new CFError({
                    lvl: 'Crit',
                    message: 'Critical error'
                });
            } }
        }, { log });
        await expect(vr.resolve('${{test_1.test}}')).rejects.toThrowError('Critical error');
        expect(log.info).toHaveBeenCalledTimes(1);
        expect(log.warn).toHaveBeenCalledTimes(1);
    });

    it('Should returned resolved=false in-case an error that is no', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'test_1': { resolve: () => {
                throw utils.buildError(errors.Type.APIRequestError, new Error('some api error'));
            } }
        }, { log });
        const res = await vr.resolve('${{test_1.test}}');
        expect(res[0].resolved).toBeFalsy();
        expect(log.info).toHaveBeenCalledTimes(1);
        expect(log.warn).toHaveBeenCalledTimes(1);
    });

    it('Should resolve with empty string in case of single resolution timedout', async () => {
        const vr = resolver.build({
            'test_1': { resolve: () => {
                return Promise.resolve().delay(30).then(() => ('value'));
            } }
        }, { singleResolutionTimeout: 20 });
        const expected = [
            {
                index: 0,
                key: '${{test_1.test}}',
                value: '',
                resolved: true,
            }
        ];
        await expect(vr.resolve('${{test_1.test}}')).resolves.toEqual(expected);
    });
});

describe('Resolve secret', () => {
    it('Should throw an error in case the name of the secret store was not provided', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'secret': secretStoreResolver.build(),
        });
        // eslint-disable-next-line max-len
        const message = 'Failed to resolve variable: ${{secret}}. Invalid reference to secret. No secret name was given, supported format "secret.NAME.KEY..."';
        await expect(vr.resolve('${{secret}}', { log })).resolves.toEqual([
            {
                index: 0,
                key: '${{secret}}',
                resolved: false,
                value: '${{secret}}'
            }
        ]);
        expect(log.warn).toHaveBeenCalledWith(message);
    });

    it('Should throw an error in case the key of the secret store was not provided', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'secret': secretStoreResolver.build(),
        });
        // eslint-disable-next-line max-len
        const message = 'Failed to resolve variable: ${{secret.name}}. Invalid reference to secret. no key reference was given, supported format: "secret.NAME.KEY..."';
        await expect(vr.resolve('${{secret.name}}', { log })).resolves.toEqual([
            {
                index: 0,
                key: '${{secret.name}}',
                resolved: false,
                value: '${{secret.name}}'
            }
        ]);
        expect(log.warn).toHaveBeenCalledWith(message);
    });

    it('Should log a suggestion in case it was provided on the error', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'secret': {
                resolve() {
                    throw new CFError({
                        message: 'Error',
                        suggestion: 'Do something',
                    });
                }
            },
        });
        await expect(vr.resolve('${{secret.name.key}}', { log })).resolves.toEqual([
            {
                index: 0,
                key: '${{secret.name.key}}',
                resolved: false,
                value: '${{secret.name.key}}'
            }
        ]);
        expect(log.warn).toHaveBeenCalledTimes(2);
        expect(log.warn).toHaveBeenNthCalledWith(2, 'Do something');
    });

    it('Should resolve default value in case a suggestion exists', async () => {
        const log = getLogger();
        const vr = resolver.build({
            'secret': {
                resolve() {
                    throw new CFError({
                        message: 'Error',
                        suggestion: 'Do something',
                        default: ''
                    });
                }
            },
        });
        await expect(vr.resolve('${{secret.name.key}}', { log })).resolves.toEqual([
            {
                index: 0,
                key: '${{secret.name.key}}',
                resolved: true,
                value: ''
            }
        ]);
    });

    it('Should return the variable as is in case it was not in Vars format ${{.*}}', async () => {
        const vr = secretStoreResolver.build();
        await expect(vr.resolve('secret')).resolves.toEqual('secret');
    });
    it('Should request the context from Codefresh', async () => {
        const vr = secretStoreResolver.build({
            'test': {
                resolve: () => ('resolved'),
            },
        });
        const log = getLogger();
        const getContext = getContextFn(async () => ({
            spec: {
                type: 'test',
            }
        }));
        await vr.resolve('${{test.key.value}}', getContext, log);
        expect(getContext).toHaveBeenCalledTimes(1);
        expect(getContext).toHaveBeenCalledWith('secret-store', { context: { name: 'key' }, decrypt: true }, log);
    });

    it('Should throw an error in case a resolver was not found for secret', async () => {
        const log = getLogger();
        const getContext = getContextFn(async () => ({
            spec: {
                type: 'test',
            }
        }));
        const message = 'Cannot resolve key: "value" from secret "key", resolver for type: "test" is not found';
        const vr = secretStoreResolver.build();
        await expect(vr.resolve('${{test.key.value}}', getContext, log)).rejects.toThrow(message);
    });

    it('Should throw an error in case the request to prepare a context failed', async () => {
        const log = getLogger();
        const getContext = getContextFn(async () => Promise.reject('http-error'));
        const message = 'Failed to get context';
        const vr = secretStoreResolver.build();
        await expect(vr.resolve('${{test.key.value}}', getContext, log)).rejects.toThrow(message);
    });

    it('Should throw an not-found error in case the request to prepare a context failed with not found error', async () => {
        const log = getLogger();
        const getContext = getContextFn(async () => Promise.reject('not found'));
        const message = 'Integration not found. Failed to prepare context: key';
        const vr = secretStoreResolver.build();
        await expect(vr.resolve('${{test.key.value}}', getContext, log)).rejects.toThrow(message);
    });

    it('Should throw an timeout error in case the request to prepare a context failed with timoue error', async () => {
        const log = getLogger();
        const getContext = getContextFn(async () => Promise.reject('ETIMEDOUT'));
        const message = 'Context request timeout. Failed to prepare context: key';
        const vr = secretStoreResolver.build();
        await expect(vr.resolve('${{test.key.value}}', getContext, log)).rejects.toThrow(message);
    });

    describe('Kubernetes secret', () => {
        it('Should infer resolver from context type', async () => {
            const log = getLogger();
            const getContext = getContextFn(async () => ({
                spec: {
                    type: 'secret-store.kubernetes',
                    data: {
                        resourceType: 'secret',
                        cluster: 'test'
                    }
                }
            }));

            const vr = resolver.build({
                'secret': secretStoreResolver.build({
                    'kubernetes-secret': {
                        resolve: () => ('resolved'),
                    }
                }),
            }, { getContext, log });
            const expected = [
                {
                    index: 0,
                    key: '${{secret.key.value}}',
                    value: 'resolved',
                    resolved: true,
                },
            ];
            await expect(vr.resolve('${{secret.key.value}}', getContext, log)).resolves.toEqual(expected);
        });

        it('Should infer resolver from context type when context type=secret-store.kubernetes-runtime', async () => {
            const log = getLogger();
            const getContext = getContextFn(async () => ({
                spec: {
                    type: 'secret-store.kubernetes-runtime',
                    data: {
                        resourceType: 'secret',
                        cluster: 'test'
                    }
                }
            }));
            const vr = resolver.build({
                'secret': secretStoreResolver.build({
                    'kubernetes-runtime-secret': {
                        resolve: () => ('resolved'),
                    }
                }),
            }, { getContext, log });
            const expected = [
                {
                    index: 0,
                    key: '${{secret.key.value}}',
                    value: 'resolved',
                    resolved: true,
                },
            ];
            await expect(vr.resolve('${{secret.key.value}}')).resolves.toEqual(expected);
        });

        it('Should infer resolver from context type when context type=secret-store.hashicorp-vault', async () => {
            const log = getLogger();
            const getContext = getContextFn(async () => ({
                spec: {
                    type: 'secret-store.hashicorp-vault',
                    data: {
                        resourceType: 'secret',
                        cluster: 'test'
                    }
                }
            }));
            const vr = resolver.build({
                'secret': secretStoreResolver.build({
                    'hashicorp-vault': {
                        resolve: () => ('resolved'),
                    }
                }),
            }, { getContext, log });
            const expected = [
                {
                    index: 0,
                    key: '${{secret.key.value}}',
                    value: 'resolved',
                    resolved: true,
                },
            ];
            await expect(vr.resolve('${{secret.key.value}}')).resolves.toEqual(expected);
        });

        it('Should get the context for the second time from the workflowContext instead of sending event', async () => {
            const log = getLogger();
            const getContext = getContextFn(async () => ({
                spec: {
                    type: 'secret-store.kubernetes',
                    data: {
                        resourceType: 'secret',
                        cluster: 'test'
                    }
                }
            }));
            const vr = resolver.build({
                'secret': secretStoreResolver.build({
                    'kubernetes-secret': {
                        resolve: (ctx, wfctx, key) => {
                            return `${key}-resolved`;
                        },
                    }
                }),
            }, { getContext, log });
            const expected = [
                {
                    index: 0,
                    key: '${{secret.azure.key}}',
                    value: 'key-resolved',
                    resolved: true,
                },
                {
                    index: 1,
                    key: '${{secret.azure.key}}',
                    value: 'key-resolved',
                    resolved: true,
                },
            ];
            await expect(vr.resolve('${{secret.azure.key}}-${{secret.azure.key}}')).resolves.toEqual(expected);
            await expect(vr.resolve('${{secret.azure.key}}')).resolves.toEqual([expected[0]]);
            expect(getContext).toHaveBeenCalledTimes(3);
        });
    });

});

describe('Recursive object resolution', () => {
    it('Should return empty array when trying to resolve undefined like object', async () => {
        const vr = resolver.build();
        const expected = [];
        await expect(vr.recursive(undefined)).resolves.toEqual(expected);
    });

    it('Should return empty array when trying to resolve an empty array', async () => {
        const vr = resolver.build();
        const expected = [];
        await expect(vr.recursive([])).resolves.toEqual(expected);
    });
    it('Should resolve just string ', async () => {
        const vr = resolver.build({
            'resolver': { resolve: () => ('resolved') },
        }, { log: getLogger() });
        const expected = [
            {
                path: '',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver}}',
                        value: 'resolved',
                        resolved: true,
                    }
                ],
            },
        ];
        await expect(vr.recursive('${{resolver}}')).resolves.toEqual(expected);
    });
    it('Should resolve array ', async () => {
        const vr = resolver.build({
            'resolver1': { resolve: () => ('resolved') },
        });
        const arr = [
            '${{resolver1}}',
            '${{resolver2}}',
        ];
        const expected = [
            {
                path: '[0]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver1}}',
                        value: 'resolved',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[1]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver2}}',
                        value: '${{resolver2}}',
                        resolved: false,
                    }
                ],
            },
        ];
        await expect(vr.recursive(arr)).resolves.toEqual(expected);
    });
    it('Should resolve all supported variables in simple key-value object', async () => {
        const obj = {
            key1: 'value1',
            key2: '${{resolver_2}}',
            key3: '${{resolver_3}}',
            key4: '${{resolver_4}}',
        };

        const vr = resolver.build({
            'resolver_2': { resolve: () => ('resolved_2') },
            'resolver_3': { resolve: () => ('resolved_3') }
        });
        const expected = [
            {
                path: '[key1]',
                resolution: [
                    {
                        index: 0,
                        key: 'value1',
                        value: 'value1',
                        resolved: false,
                    }
                ],
            },
            {
                path: '[key2]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver_2}}',
                        value: 'resolved_2',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[key3]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver_3}}',
                        value: 'resolved_3',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[key4]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver_4}}',
                        value: '${{resolver_4}}',
                        resolved: false,
                    }
                ],
            },
        ];
        await expect(vr.recursive(obj)).resolves.toEqual(expected);
    });

    it('Should resolve in nasted object', async () => {
        const obj = {
            key1: {
                key2: '${{resolver}}'
            },
            key3: '${{resolver}}',
            key4: {
                key5: {
                    key6: '${{no_resolver}}',
                },
            }
        };

        const vr = resolver.build({
            'resolver': { resolve: () => ('resolved') },
        });
        const expected = [
            {
                path: '[key1][key2]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver}}',
                        value: 'resolved',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[key3]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver}}',
                        value: 'resolved',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[key4][key5][key6]',
                resolution: [
                    {
                        index: 0,
                        key: '${{no_resolver}}',
                        value: '${{no_resolver}}',
                        resolved: false,
                    }
                ],
            },
        ];
        await expect(vr.recursive(obj)).resolves.toEqual(expected);
    });

    it('Should resolve in nasted array', async () => {
        const arr = [
            '${{resolver}}',
            [
                '${{resolver}}',
                '${{no_resolver}}',
            ],
        ];
        const expected = [
            {
                path: '[0]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver}}',
                        value: 'resolved',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[1][0]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver}}',
                        value: 'resolved',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[1][1]',
                resolution: [
                    {
                        index: 0,
                        key: '${{no_resolver}}',
                        value: '${{no_resolver}}',
                        resolved: false,
                    }
                ],
            },
        ];
        const vr = resolver.build({
            'resolver': { resolve: () => ('resolved') },
        });
        await expect(vr.recursive(arr)).resolves.toEqual(expected);
    });

    it('codefresh.yaml resolution', async () => {
        const obj = {
            steps: {
                step1: {
                    image: '${{CF_IMAGE}}',
                    commands: [
                        'echo ${{resolver_1}} && echo ${{resolver_2}}',
                        'echo ${{resolver_3}}',
                    ],
                    when: {
                        steps: [
                            {
                                name: '${{CF_STEP}}',
                                on: [
                                    'success'
                                ]
                            }
                        ]
                    }
                }
            }
        };

        const vr = resolver.build({
            'resolver_1': { resolve: () => ('resolved value 1') },
            'resolver_2': { resolve: () => ('resolved value 2') },
            'resolver_3': { resolve: () => ('resolved value 3') },
        });
        const expected = [
            {
                path: '[steps][step1][image]',
                resolution: [
                    {
                        index: 0,
                        key: '${{CF_IMAGE}}',
                        value: '${{CF_IMAGE}}',
                        resolved: false
                    }
                ],
            },
            {
                path: '[steps][step1][commands][0]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver_1}}',
                        value: 'resolved value 1',
                        resolved: true
                    },
                    {
                        index: 1,
                        key: '${{resolver_2}}',
                        value: 'resolved value 2',
                        resolved: true
                    }
                ]
            },
            {
                path: '[steps][step1][commands][1]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver_3}}',
                        value: 'resolved value 3',
                        resolved: true
                    },
                ]
            },
            {
                path: '[steps][step1][when][steps][0][name]',
                resolution: [
                    {
                        index: 0,
                        key: '${{CF_STEP}}',
                        value: '${{CF_STEP}}',
                        resolved: false
                    },
                ]
            },
            {
                path: '[steps][step1][when][steps][0][on][0]',
                resolution: [
                    {
                        index: 0,
                        key: 'success',
                        value: 'success',
                        resolved: false
                    },
                ]
            },
        ];
        await expect(vr.recursive(obj)).resolves.toEqual(expected);
    });

    it('should not fail in case resolver throws an error', async () => {
        const obj = {
            key2: '${{resolver_2}}',
            key3: '${{resolver_3}}',
        };

        const vr = resolver.build({
            'resolver_2': { resolve: () => ('resolved_2') },
            'resolver_3': { resolve: async () => { throw new Error('failed'); } }
        });
        const expected = [
            {
                path: '[key2]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver_2}}',
                        value: 'resolved_2',
                        resolved: true,
                    }
                ],
            },
            {
                path: '[key3]',
                resolution: [
                    {
                        index: 0,
                        key: '${{resolver_3}}',
                        value: '${{resolver_3}}',
                        resolved: false,
                    }
                ],
            },
        ];
        await expect(vr.recursive(obj)).resolves.toEqual(expected);
    });
});

describe('utils', () => {
    describe('split into parts', () => {
        it('should return the name of the secret store and the variable', () => {
            expect(split('secrets.name.key')).toEqual(['secrets', 'name', 'key']);
        });
        it('should return key with dots', () => {
            expect(split('secrets.name.key.with.dots')).toEqual(['secrets', 'name', 'key.with.dots']);
        });
    });
});
