const _ = require('lodash');
const vault = require('node-vault');
const CFError = require('cf-errors');
const { buildError, to } = require('../utils');
const { Type: { InitError, AuthenticationError, MissingReferenceError } } = require('../errors');

const DEFAULT_OPTIONS = {
    apiVersion: 'v1',
    requestOptions: {
        followAllRedirects: true,
        // TODO: add strict ssl indicator
    }
};

class Vault {
    constructor(apiUrl, auth) {
        if (!apiUrl) {
            throw buildError(InitError, null, 'Missing apiUrl value for Vault');
        }

        if (_.isEmpty(auth)) {
            throw buildError(AuthenticationError, null, 'Missing auth information for Vault');
        }

        this._apiUrl = apiUrl;
        this._auth = auth;
    }

    /**
     * Initializes the vault client
     */
    async init() {
        const { _auth: { token, type } } = this;
        this._client = vault({
            ...DEFAULT_OPTIONS,
            endpoint: this._apiUrl,
            ...(token && { token }),
        });
        let testFunc;
        if (type === 'token') {
            testFunc = this._client.tokenLookupSelf.bind(this.client);
        } else if (this._client[`${type}Login`]) {
            testFunc = this._client[`${type}Login`].bind(this._client, this._auth);
        } else {
            throw buildError(AuthenticationError, null, `Auth method '${type}' is not supported`);
        }

        const [err] = await to(testFunc());
        if (err) {
            throw buildError(AuthenticationError, err, 'Authentication failed');
        }
    }

    async read(path) {
        const [err, res] = await to(this._client.read(path));
        if (err) {
            const { errors, warnings } = _.get(err, 'response.body', {});
            const error = errors && errors[0];
            const warning = warnings && warnings[0];

            if (error || warning) {
                throw new CFError({
                    message: `Failed to read secret with: "${error || warning}".`,
                    statusCode: _.get(err, 'response.statusCode'),
                });
            }

            throw buildError(MissingReferenceError, err);
        }
        return res;
    }

    /**
     * secret reference
     * @typedef {Object} SecretReference
     * @property {string} path
     * @property {string} key
     */

    /**
     * finds the path and key from a context and a variable string
     * @param {any} ctx vault secret-store context
     * @param {string | undefined} variable variable string
     * @returns {SecretReference} secret reference
     */
    static getReferences(ctx, variable = '') {
        let path = _.get(ctx, 'spec.data.path');
        let key = '';
        if (path && variable.includes('@')) {
            const suggestion = 'The specified context is locked to a specific path and cannot be use to resolve secrets from a different path';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        if (!variable.includes('@')) {
            const suggestion = 'The specified context is not locked to a specific, the secret reference should be using the format: "/path/to/secret@key-name"';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        ([path, key] = variable.split('@'));
        if (!path || !key) {
            const suggestion = 'The reference is missing a path to the specified secret or a secret key, make sure you use the correct format: "/path/to/secret@key-name"';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        return {
            path,
            key,
        };
    }
}

module.exports = Vault;
