const Promise = require('bluebird');
const _ = require('lodash');
const { readFile } = require('fs');
const { Client } = require('kubernetes-client');
const Request = require('kubernetes-client/backends/request');
const KubeConfigBuilder = require('kube-config-builder');
const errors = require('../errors');
const { buildError, to } = require('../utils');


async function _buildClientWithConfig(cluster) {
    const cnf = await KubeConfigBuilder.Builder.ConstructKubeAPIConfigJson(cluster);
    const client = new Client({ backend: new Request(cnf) });
    await client.loadSpec();
    return client;
}

async function _buildClientInCluster() {
    const backend = new Request(Request.config.getInCluster());
    const client = new Client({ backend });
    await client.loadSpec();
    return client;
}

async function _buildKubernetesClient({ cluster, getContext, log }) {
    let clusterConfig;
    if (cluster) {
        // get the cluster integration
        const _clusterConfig = await getContext('kubernetes-cluster', {
            selector: cluster
        }, log);
        clusterConfig = _clusterConfig;
    }

    // build the client
    return clusterConfig ? _buildClientWithConfig(clusterConfig) : _buildClientInCluster();
}

/**
 * build suggestion
 * @param {{ cluster: string, namespace: string, name: string, type: string, key: string }} params params
 * @returns {string} suggestion
 */
function buildSuggestion({ cluster, namespace, name, type, key }) {
    if (cluster) {
        return `Run command "kubectl --namespace ${namespace} --cluster ${cluster} get ${type} ${name} -o yaml", make sure key "${key}" is exist`;
    }
    return `Run command "kubectl --namespace ${namespace} get ${type} ${name} -o yaml", make sure key "${key}" is exist`;
}

/**
 * construct a k8s client
 * @param {{ cluster: string, getContext: getContextFn, log: Logger}} params params
 */
async function buildClient({ cluster, getContext, log }) {
    const [err, client] = await to(_buildKubernetesClient({ cluster, getContext, log }));
    if (err) {
        if (_.includes(err.message, 'not found')) {
            throw buildError(errors.Type.IntegrationNotFoundError, err);
        }
        if (_.includes(err.message, 'ETIMEDOUT')) {
            throw buildError(errors.Type.RequestTimeoutError, err);
        }
        const suggestion = `Make sure your Kubernetes cluster "${cluster}" exists and the Kubernetes API is responsive`;
        throw buildError(errors.Type.APIRequestError, err, suggestion);
    }
    return client;
}

async function getCurrentNamespace() {
    const location = '/var/run/secrets/kubernetes.io/serviceaccount/namespace';
    const ns = await Promise.fromCallback(cb => readFile(location, 'utf8', cb));
    return ns;
}

function getReferences(ctx, variable = '') {
    const namespace = _.get(ctx, 'spec.data.namespace');
    let name = '';
    let key = '';
    if (variable.includes('@')) {
        const parts = variable.split('@');
        name = _.head(parts);
        key = _.tail(parts).join('@');
    } else if (_.has(ctx, 'spec.data.resourceName')) {
        name = ctx.spec.data.resourceName;
        key = variable;
    } else {
        const suggestion = 'Make sure the context has reference to resoureceName or the variable format is resource-name@key';
        throw buildError(errors.Type.MissingReferenceError, undefined, suggestion);
    }
    return {
        namespace,
        name,
        key,
    };
}

module.exports = {
    buildSuggestion,
    buildClient,
    getCurrentNamespace,
    getReferences
};
