const _ = require('lodash');
const CFError = require('cf-errors');
const logger = require('cf-logs').Logger('codefresh:variable-resolver:resolvers:secret-store');
const { split, buildError, to } = require('../../utils');
const Vars = require('../../Vars');
const errors = require('../../errors');

/**
 * builds a secret resolver
 * @param {Object.<string, Resolver>} resolvers
 * @returns {Resolver & { secret: true }} secretResolver
 */
function build(resolvers) {
    return {
        resolve: resolve.bind({ resolvers }),
        secret: true,
    };
}

/**
 * resolver function
 * @this {{resolvers: Object.<string, Resolver>}}
 * @param {string} variable - the secret to resolve
 * @param {getContextFn} getContext - to request for contexts
 * @param {Logger} log - logger
 * @returns {Promise<string>} resolvedValue
 */
async function resolve(variable, getContext, log) {
    const vars = new Vars(variable);
    if (!vars.valueContainsVariables()) {
        return variable;
    }
    const resolvers = this.resolvers;
    const [, name, key] = split(_.head(vars.getVariableKeys()));

    logger.info(`Resolving ${variable}`);
    if (!name) {
        throw new CFError('Invalid reference to secret. No secret name was given, supported format "secret.NAME.KEY..."');
    }
    if (!key) {
        throw new CFError('Invalid reference to secret. no key reference was given, supported format: "secret.NAME.KEY..."');
    }

    const [err, ctx] = await to(async () => _requestContext(name, getContext, log, true));
    if (err) {
        if (_.includes(err.toString(), 'not found')) {
            throw buildError(errors.Type.IntegrationNotFoundError, err);
        }
        if (_.includes(err.toString(), 'ETIMEDOUT')) {
            throw buildError(errors.Type.ContextRequestTimeoutError, err);
        }
        throw buildError(errors.Type.ContextRequestError, err);
    }
    logger.info(`Got context ${name}`);

    const type = _inferTypeFromContext(ctx);
    if (!resolvers || !resolvers[type]) {
        throw new CFError(`Cannot resolve key: "${key}" from secret "${name}", resolver for type: "${type}" is not found`);
    }

    return resolvers[type].resolve(ctx, getContext, key, log);
}

/**
 * Fetches a context object
 * @param {string} name name of the context
 * @param {getContextFn} getContext function that gets the context
 * @param {Logger} log logger
 * @param {boolean} decrypt should it get the decrypted context
 * @returns {*} context
 */
async function _requestContext(name, getContext, log, decrypt = false) {
    logger.info(`Requesting ${name} secret-store data`);
    const [err, context] = await to(getContext('secret-store', {
        context: {
            name,
        },
        decrypt,
    }, log));
    if (err) {
        throw new CFError({
            cause: err,
            message: `Failed to prepare context: ${name}`,
        });
    }
    return context;
}

/**
 * returns the context's type
 * @param {*} context context object
 * @returns {string} type
 */
function _inferTypeFromContext(context) {
    const type = _.get(context, 'spec.type', '');
    if (type === 'secret-store.kubernetes') {
        return `kubernetes-${context.spec.data.resourceType}`;
    }
    if (type === 'secret-store.kubernetes-runtime') {
        return `kubernetes-runtime-${context.spec.data.resourceType}`;
    }
    if (type === 'secret-store.hashicorp-vault') {
        return `hashicorp-vault`;
    }

    return type;
}

module.exports = { build };
