/* eslint-disable prefer-arrow-callback */
jest.mock('fs');
jest.mock('kube-config-builder');
jest.mock('kubernetes-client');
jest.mock('kubernetes-client/backends/request');

const Promise = require('bluebird');
const fs = require('fs');
const errors = require('../../../../errors');
const { buildError, to } = require('../../../../utils');
const { getReferences } = require('../../../../clients/kubernetes');
const {
    kubernetesConfigmap,
    kubernetesSecert,
    kubernetesRuntimeConfigmap,
    kubernetesRuntimeSecert,
} = require('../');
const k8sClient = require('kubernetes-client');

const buildK8SContext = (resourceType) => {
    return {
        spec: {
            data: {
                cluster: 'test-cluster',
                namespace: 'default',
                resourceName: 'test-resource',
                resourceType,
            }
        }
    };
};

const buildSecretStore = (cluster) => {
    return {
        spec: {
            data: {
                cluster,
            }
        }
    };
};

const buildRuntimeK8SContext = (resourceType) => {
    return {
        spec: {
            data: {
                resourceName: 'test-resource',
                resourceType,
            }
        }
    };
};

const buildKubeAPI = (getResponse = Promise.resolve) => {
    return {
        v1: {
            namespace() {
                return {
                    secret() {
                        return {
                            get: getResponse,
                        };
                    },
                    configmap() {
                        return {
                            get: getResponse
                        };
                    }
                };
            }
        }
    };
};

const buildClient = (api) => {
    return {
        loadSpec: () => Promise.resolve(),
        api,
    };
};

const getContextFn = mock => jest.fn(mock);
const logger = {
    info: () => {},
    warn: () => {},
};

describe('build kube client', () => {
    describe('throw', () => {
        it('APIRequestError in case failed to build kubeclient for unknwon reason', async () => {
            const suggestion = 'Make sure your Kubernetes cluster "asd" exists and the Kubernetes API is responsive';
            const expected = buildError(errors.Type.APIRequestError, new Error(''), suggestion);
            const [err] = await to(kubernetesSecert.resolve(buildSecretStore('asd'), getContextFn(async () => { throw new Error(''); }), '', logger));
            expect(err.message).toEqual(expected.message);
            expect(err.lvl).toEqual(expected.lvl);
            expect(err.suggestion).toEqual(expected.suggestion);
            expect(err.cause).toEqual(expected.cause);
        });
        it('IntegrationNotFoundError in case the secret store was not found', async () => {
            const expected = buildError(errors.Type.IntegrationNotFoundError, new Error('not found'));
            const [err] = await to(kubernetesSecert.resolve(buildSecretStore('asd'), getContextFn(async () => { throw new Error('not found'); }), '', logger));
            expect(err.message).toEqual(expected.message);
            expect(err.lvl).toEqual(expected.lvl);
            expect(err.suggestion).toEqual(expected.suggestion);
            expect(err.cause).toEqual(expected.cause);
        });
        it('RequestTimeoutError in case the request to Codefresh timedout', async () => {
            const expected = buildError(errors.Type.RequestTimeoutError, new Error('ETIMEDOUT'));
            const [err] = await to(kubernetesSecert.resolve(buildSecretStore('asd'), getContextFn(async () => { throw new Error('ETIMEDOUT'); }), '', logger));
            expect(err.message).toEqual(expected.message);
            expect(err.lvl).toEqual(expected.lvl);
            expect(err.suggestion).toEqual(expected.suggestion);
            expect(err.cause).toEqual(expected.cause);
        });
    });
});

describe('Should resolve secret store type=kubernetes ', () => {
    describe('when resource type=secret', () => {
        it('should an error in case failed to get secrets from API server', async () => {
            const ctx = buildK8SContext('secret');
            const kubeapi = buildKubeAPI(() => Promise.reject(new Error('Error')));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const expected = new Error('Failed to request API server');
            expected.cause = new Error('Error');
            expected.lvl = 'Warn';
            expected.message = `${expected.message}. ${expected.cause.message}`;
            const [err] = await to(kubernetesSecert.resolve(ctx, getContextFn(() => Promise.resolve()), '', logger));
            expect(err.message).toEqual(expected.message);
            expect(err.cause).toEqual(expected.cause);
            expect(err.lvl).toEqual(expected.lvl);
        });

        it('should throw an error in case the key was not found with suggestion', async () => {
            const ctx = buildK8SContext('secret');
            const kubeapi = buildKubeAPI(() => Promise.resolve());
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const [err] = await to(kubernetesSecert.resolve(ctx, getContextFn(async () => kubeapi), 'not-found'));
            expect(err.message).toEqual('Key not found');
            const expected = 'Run command "kubectl --namespace default --cluster test-cluster get secret test-resource -o yaml", make sure key "not-found" is exist';
            expect(err.suggestion).toEqual(expected);
        });
        it('should return decoded key', async () => {
            const ctx = buildK8SContext('secret');
            const kubeapi = buildKubeAPI(async () => ({
                body: {
                    data: {
                        secret: 'Zm91bmQta2V5'
                    }
                }
            }));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            await expect(kubernetesSecert.resolve(ctx, getContextFn(), 'secret')).resolves.toEqual('found-key');
        });
    });
    describe('when resource type=configmap', () => {
        it('should an error in case failed to get secrets from API server', async () => {
            const ctx = buildK8SContext('configmap');
            const kubeapi = buildKubeAPI(() => Promise.reject(new Error('Error')));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const expected = new Error('Failed to request API server');
            expected.cause = new Error('Error');
            expected.lvl = 'Warn';
            expected.message = `${expected.message}. ${expected.cause.message}`;
            const [err] = await to(kubernetesConfigmap.resolve(ctx, getContextFn(), '', logger));
            expect(err.message).toEqual(expected.message);
            expect(err.cause).toEqual(expected.cause);
            expect(err.lvl).toEqual(expected.lvl);
        });
        it('should throw an error in case the key was not found with suggestion', async () => {
            const ctx = buildK8SContext('configmap');
            const kubeapi = buildKubeAPI(() => Promise.resolve());
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const [err] = await to(kubernetesConfigmap.resolve(ctx, getContextFn(), 'not-found'));
            expect(err.message).toEqual('Key not found');
            const expected = 'Run command "kubectl --namespace default --cluster test-cluster get configmap test-resource -o yaml", make sure key "not-found" is exist';
            expect(err.suggestion).toEqual(expected);
        });
        it('should return key', async () => {
            const ctx = buildK8SContext('configmap');
            const kubeapi = buildKubeAPI(async () => ({
                body: {
                    data: {
                        key: 'value'
                    }
                }
            }));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            await expect(kubernetesConfigmap.resolve(ctx, getContextFn(), 'key', logger)).resolves.toEqual('value');
        });
    });
});

describe('Should resolve secret store type=kubernetes-runtime', () => {
    describe('when resource type=secret', () => {
        it('Should throw an error in case the namespace file was not found', async () => {
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(new Error('not found'));
            });
            const ctx = buildRuntimeK8SContext('secret');
            const [err] = await to(kubernetesRuntimeSecert.resolve(ctx, undefined, 'some-key'));
            expect(err.message).toEqual('Failed to read local filesystem. not found');
            const expected = 'Make sure namespace file exists on local FS';
            expect(err.suggestion).toEqual(expected);
        });
        it('should an error in case failed to get secrets from API server', async () => {
            const ctx = buildRuntimeK8SContext('secret');
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(null, 'default');
            });
            const kubeapi = buildKubeAPI(() => Promise.reject(new Error('Error')));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const expected = new Error('Failed to request API server');
            expected.cause = new Error('Error');
            expected.lvl = 'Warn';
            expected.message = `${expected.message}. ${expected.cause.message}`;
            const [err] = await to(kubernetesRuntimeSecert.resolve(ctx, getContextFn(), '', logger));
            expect(err.message).toEqual(expected.message);
            expect(err.cause).toEqual(expected.cause);
            expect(err.lvl).toEqual(expected.lvl);
        });
        it('should throw an error in case the key was not found with suggestion', async () => {
            const ctx = buildRuntimeK8SContext('secret');
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(null, 'default');
            });
            const kubeapi = buildKubeAPI(() => Promise.resolve());
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const [err] = await to(kubernetesRuntimeSecert.resolve(ctx, getContextFn(), 'not-found', logger));
            expect(err.message).toEqual('Key not found');
            const expected = 'Run command "kubectl --namespace default get secret test-resource -o yaml", make sure key "not-found" is exist';
            expect(err.suggestion).toEqual(expected);
        });
        it('should return key', async () => {
            const ctx = buildRuntimeK8SContext('secret');
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(null, 'default');
            });
            const kubeapi = buildKubeAPI(async () => ({
                body: {
                    data: {
                        key: 'Zm91bmQta2V5'
                    }
                }
            }));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            await expect(kubernetesRuntimeSecert.resolve(ctx, getContextFn(), 'key', logger)).resolves.toEqual('found-key');
        });
    });

    describe('when resource type=configmap', () => {
        it('Should throw an error in case the namespace file was not found', async () => {
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(new Error('not found'));
            });
            const ctx = buildRuntimeK8SContext('configmap');
            const [err] = await to(kubernetesRuntimeConfigmap.resolve(ctx, undefined, 'some-key'));
            expect(err.message).toEqual('Failed to read local filesystem. not found');
            const expected = 'Make sure namespace file exists on local FS';
            expect(err.suggestion).toEqual(expected);
        });
        it('should an error in case failed to get configmap from API server', async () => {
            const ctx = buildRuntimeK8SContext('configmap');
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(null, 'default');
            });
            const kubeapi = buildKubeAPI(() => Promise.reject(new Error('Error')));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const expected = new Error('Failed to request API server');
            expected.cause = new Error('Error');
            expected.lvl = 'Warn';
            expected.message = `${expected.message}. ${expected.cause.message}`;
            const [err] = await to(kubernetesRuntimeConfigmap.resolve(ctx, getContextFn(), '', logger));
            expect(err.message).toEqual(expected.message);
            expect(err.cause).toEqual(expected.cause);
            expect(err.lvl).toEqual(expected.lvl);
        });
        it('should throw an error in case the key was not found with suggestion', async () => {
            const ctx = buildRuntimeK8SContext('configmap');
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(null, 'default');
            });
            const kubeapi = buildKubeAPI(() => Promise.resolve());
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            const [err] = await to(kubernetesRuntimeConfigmap.resolve(ctx, getContextFn(), 'not-found', logger));
            expect(err.message).toEqual('Key not found');
            const expected = 'Run command "kubectl --namespace default get configmap test-resource -o yaml", make sure key "not-found" is exist';
            expect(err.suggestion).toEqual(expected);
        });
        it('should return key', async () => {
            const ctx = buildRuntimeK8SContext('configmap');
            fs.readFile.mockImplementationOnce((path, options, cb) => {
                cb(null, 'default');
            });
            const kubeapi = buildKubeAPI(async () => ({
                body: {
                    data: {
                        key: 'found-key'
                    }
                }
            }));
            k8sClient.Client.mockImplementationOnce(function () { return buildClient(kubeapi); });
            await expect(kubernetesRuntimeConfigmap.resolve(ctx, getContextFn(), 'key', logger)).resolves.toEqual('found-key');
        });
    });
});

describe('references', () => {
    const resource = 'resource';
    const namespace = 'default';
    const keyName = 'key';
    describe('should resolve', () => {
        it('namespace from context, resource-name from context, key from variable', () => {
            const ctx = {
                spec: {
                    data: {
                        namespace,
                        resourceName: resource
                    }
                }
            };
            const variable = keyName;
            expect(getReferences(ctx, variable)).toEqual({
                namespace,
                name: resource,
                key: keyName
            });
        });

        it('namespace from context, resource-name exist on context and on variable (prefer variable), key from variable', () => {
            const ctx = {
                spec: {
                    data: {
                        namespace,
                        resourceName: resource
                    }
                }
            };
            const variable = `resourece-overwrite@${keyName}`;
            expect(getReferences(ctx, variable)).toEqual({
                namespace,
                name: 'resourece-overwrite',
                key: keyName
            });
        });
        it('namespace from context, resource-name from variable, key from variable', () => {
            const ctx = {
                spec: {
                    data: {
                        namespace,
                    }
                }
            };
            const variable = `${resource}@${keyName}`;
            expect(getReferences(ctx, variable)).toEqual({
                namespace,
                name: resource,
                key: keyName
            });
        });
    });
    describe('should thorw error', () => {
        it('when the context is missing resource-name and the variable doest have it as well', async () => {
            const ctx = {
                spec: {
                    data: {
                        namespace,
                    }
                }
            };
            const variable = keyName;
            const expected = {
                message: 'Failed to resolve the required reference',
                lvl: 'Crit',
                suggestion: 'Make sure the context has reference to resoureceName or the variable format is resource-name@key'
            };
            const [err] = await to(async () => getReferences(ctx, variable));
            expect(err.message).toEqual(expected.message);
            expect(err.lvl).toEqual(expected.lvl);
            expect(err.suggestion).toEqual(expected.suggestion);
        });
    });
});
